/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.base.ui.annotation.valueprovider.IAnnotationValueProvider;
import org.fortiss.tooling.kernel.model.IProjectRootElement;
import org.fortiss.tooling.kernel.service.base.EObjectAwareServiceBase;
import org.fortiss.tooling.kernel.utils.EcoreUtils;
import org.fortiss.tooling.kernel.utils.KernelModelElementUtils;

/**
 * 
 * @author eder
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 18FB575985A117D61299ECC358019379
 */
public class AnnotationValueService extends
		EObjectAwareServiceBase<IAnnotationValueProvider<IAnnotatedSpecification>> implements
		IAnnotationValueService {

	/** The compositor extension point ID. */
	private static final String EXTENSION_POINT_NAME =
			"org.fortiss.tooling.base.ui.annotationViewPart";

	/** The compositor configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "annotationViewPart";

	/** The handler class attribute name. */
	private static final String HANDLER_CLASS_ATTRIBUTE_NAME = "binding";

	/**
	 * 
	 */
	public AnnotationValueService() {
		super();
	}

	/** {@inheritDoc} */
	@Override
	public List<AnnotationEntry> getValues(final IModelElement element) {
		if(element == null) {
			return Collections.emptyList();
		}
		final List<AnnotationEntry> result = new ArrayList<AnnotationEntry>();// Collections.emptyMap();
		final List<IAnnotationValueProvider<IAnnotatedSpecification>> registeredHandlers =
				getRegisteredHandlers(element.getClass());
		if(registeredHandlers != null && !registeredHandlers.isEmpty()) {

			KernelModelElementUtils.runAsCommand(element, new Runnable() {

				@Override
				public void run() {
					EObject root = element;
					while(!(root instanceof IProjectRootElement)) {
						root = root.eContainer();
					}
					for(IAnnotationValueProvider<IAnnotatedSpecification> annotationProvider : registeredHandlers) {

						List<? extends IModelElement> allChildren =
								getAllChildren(root, element.getClass(), annotationProvider
										.excludeModelElementTypeFromAnnotatedSpecification());

						for(final IModelElement e : allChildren) {
							IAnnotatedSpecification annotatedSpecification =
									EcoreUtils.pickFirstInstanceOf(
											annotationProvider.getAnnotationClazz(),
											e.getSpecifications());
							if(annotatedSpecification == null) {
								try {
									annotatedSpecification =
											annotationProvider
													.getAnnotatedSpecificationForModelElement(e);
								} catch(Exception ex) {
									System.out
											.println("Error instantating new annotation: createAnnotatedSpecificationForModelElement() failed for " +
													annotationProvider.getAnnotationClazz() + ".");
									return;
								}
							}

							boolean add = true;
							for(AnnotationEntry entry : result) {
								if(entry.getModelElement().equals(e)) {
									entry.addNewSpecification(annotationProvider,
											annotatedSpecification);
									add = false;
								}
							}
							if(add) {
								AnnotationEntry entry = new AnnotationEntry(e);
								entry.addNewSpecification(annotationProvider,
										annotatedSpecification);
								result.add(entry);
							}

						}
					}
				}
			});
			return result;
		}
		return result;
	}

	/**
	 * Returns all children from element of the given type and filters out children of the given
	 * {@code modelElementTypesExcludedFromAnnotatedSpecification} list.
	 */
	private <S, T> List<? extends IModelElement> getAllChildren(EObject element,
			Class<? extends IModelElement> type,
			List<Class<? extends EObject>> modelElementTypesExcludedFromAnnotatedSpecification) {

		List<? extends IModelElement> results = EcoreUtils.getChildrenWithType(element, type);
		if(modelElementTypesExcludedFromAnnotatedSpecification != null) {
			List<IModelElement> toRemove = new ArrayList<IModelElement>();
			for(IModelElement modelElement : results) {
				for(Class<?> c : modelElementTypesExcludedFromAnnotatedSpecification) {
					if(c.isInstance(modelElement)) {
						toRemove.add(modelElement);
						break;
					}
				}
			}

			results.removeAll(toRemove);
		}

		return results;
	}

	/** {@inheritDoc} */
	@Override
	protected String getExtensionPointName() {
		return EXTENSION_POINT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getConfigurationElementName() {
		return CONFIGURATION_ELEMENT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getHandlerClassAttribute() {
		return HANDLER_CLASS_ATTRIBUTE_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected boolean requiresUniqueHandler() {
		return false;
	}

}
