/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.editingsupport;

import org.eclipse.jface.viewers.ColumnViewer;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.ui.annotation.AnnotationEntry;
import org.fortiss.tooling.base.ui.annotation.IAnnotationValueService;
import org.fortiss.tooling.base.ui.annotation.valueprovider.DynamicInstanceAnnotationValueProviderBase;

/**
 * This class extends {@link TextEditingSupport} to support text annotations for which multiple
 * instances can exist.
 * 
 * @see DynamicInstanceAnnotationValueProviderBase
 * 
 * @author diewald, barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class MultiInstanceAnnotationTextEditingSupport extends TextEditingSupport {

	/** Instance key to identify the corresponding instance of the annotation */
	private String instanceKey;

	/** Constructs a new {@link MultiInstanceAnnotationTextEditingSupport} for a specific column. */
	public MultiInstanceAnnotationTextEditingSupport(ColumnViewer viewer,
			Class<? extends IAnnotatedSpecification> clazz, String instanceKey) {
		super(viewer, clazz);

		this.instanceKey = instanceKey;
	}

	/** {@inheritDoc} */
	@Override
	protected Object doGetValue(AnnotationEntry annotationEntry) {
		return annotationEntry.getSpecificationValue(specClass, instanceKey);
	}

	/** {@inheritDoc} */
	@Override
	protected void doSetValue(AnnotationEntry annotationEntry, String value) throws Exception {
		annotationEntry.setSpecificationValue(value, specClass, instanceKey);
	}

	/** {@inheritDoc} */
	@Override
	protected boolean canEdit(Object element) {
		if(element instanceof AnnotationEntry) {
			AnnotationEntry data = (AnnotationEntry)element;

			// Check if a value already exists.
			if(data.getSpecificationValue(specClass, instanceKey) != null) {
				return true;
			}

			// For dynamically instantiated annotations, check if the key exists with at least one
			// model element.
			for(AnnotationEntry entry : IAnnotationValueService.INSTANCE.getValues(data
					.getModelElement())) {

				if(entry.getInstanceKeys(specClass).contains(instanceKey)) {
					return true;
				}
			}
		}

		return false;
	}
}
