/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.valueprovider;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IDerivedAnnotation;

/**
 * Base class for {@link IAnnotationValueProvider} for derived annotations (i.e., based on
 * {@link IDerivedAnnotation}).
 * 
 * @author barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public abstract class DerivedAnnotationValueProviderBase<T extends IDerivedAnnotation<?>> extends
		ValueProviderBase<T> {

	/**
	 * Constructs a {@link IAnnotationValueProvider} for derived annotations (i.e., based on
	 * {@link IDerivedAnnotation}).
	 */
	public DerivedAnnotationValueProviderBase(EClass annotatedSpecificationEClass) {
		super(annotatedSpecificationEClass);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public <U> U getAnnotationValue(T specification) {
		// Delegate retrieval of annotation value to {@link EOperation} to be provided for the
		// concrete @link DerivedAnnotationBase}.
		return (U)specification.getValue();
	}

	/** {@inheritDoc} */
	@Override
	public EditingSupport createEditingSupport(ColumnViewer viewer,
			Class<? extends IAnnotatedSpecification> clazz, String instanceKey) {

		// Derived annotations are results of calculations can thus cannot be edited.
		return null;
	}

}
