/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2013 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.compose;

import org.eclipse.gef.RootEditPart;
import org.fortiss.tooling.base.model.base.EntryConnectorBase;
import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.base.model.element.IHierarchicElement;
import org.fortiss.tooling.base.ui.dnd.gef.ConnectionDragContext;
import org.fortiss.tooling.base.ui.editpart.FreeConnectorEditPartBase;
import org.fortiss.tooling.kernel.extension.data.IConnectionCompositionContext;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * Base class for all connector - hierarchical element connections
 * 
 * @author igna
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating GREEN Hash: D0D66CFFDCDC71E6A1A6E2CC6F91527B
 */
public abstract class ConnectorHierarchicElementConnectionCompositorBase<HE extends IHierarchicElement, S extends IConnector, T extends IHierarchicElement>
		extends ConnectionCompositorBase<HE, S, T> {

	/** {@inheritDoc} */
	@Override
	public boolean canConnect(S source, T target, HE parent, IConnectionCompositionContext context) {
		if(context instanceof ConnectionDragContext) {
			ConnectionDragContext dc = (ConnectionDragContext)context;
			if(dc.getSource() instanceof FreeConnectorEditPartBase &&
					dc.getTarget() instanceof RootEditPart) {
				return canConnectInterally();
			}
		}
		return true;
	}

	/** Creates the application specific source {@link IConnector} model element. */
	protected abstract IConnector createEntryConnector();

	/** Creates the application specific target {@link IConnector} model element. */
	protected abstract IConnector createExitConnector();

	/** {@inheritDoc} */
	@Override
	public boolean connect(S source, T target, HE parent, IConnectionCompositionContext context) {
		ITopLevelElement topElement = IPersistencyService.INSTANCE.getTopLevelElementFor(parent);

		IConnector targetConnector = null;
		if(sameParent(target, source)) {
			if(source instanceof EntryConnectorBase) {
				targetConnector = createEntryConnector();
			} else {
				targetConnector = createExitConnector();
			}
		} else {
			if(source instanceof EntryConnectorBase) {
				targetConnector = createExitConnector();
			} else {
				targetConnector = createEntryConnector();
			}
		}

		avoidDuplicateConnectorName(target, targetConnector);

		if(topElement != null) {
			topElement.prepareIDs(targetConnector);
		}
		createNewTargetDropContext(target, context, targetConnector);

		createConnection(source, targetConnector, parent, topElement);
		return true;
	}

}
