/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.graphicsGL.ui.light;

import static org.lwjgl.opengl.GL11.glDisable;

import org.fortiss.tooling.graphicsGL.model.scene.Light3D;
import org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives;

/**
 * Class representing a light source based on the model object of class {@link Light3D}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 419E574748A416756B1587B03C14F918
 */
public class Light3DLight {

	/** The GL_LIGHTx to be used. */
	private final int glLightNumber;

	/** The light model. */
	private final Light3D lightModel;

	/** Constructor. */
	public Light3DLight(int glLight, Light3D lightModel) {
		this.glLightNumber = glLight;
		this.lightModel = lightModel;
	}

	/** Enables the light in GL. */
	public void enableLight() {
		GLPrimitives.enableLight(glLightNumber, lightModel);
	}

	/** Disables the light in GL. */
	public void disableLight() {
		glDisable(glLightNumber);
	}
}
