/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension.data;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.fortiss.tooling.kernel.extension.ITransformationProvider;
import org.fortiss.tooling.kernel.extension.exception.ChainTransformationFailedException;
import org.fortiss.tooling.kernel.extension.exception.TransformationFailedException;

/**
 * A transformation provider chain consists of a list of transformation
 * providers, which are able to transform objects of the given source element
 * class into an instance of the given target element class.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 710BEC527492779A8E599292CDFD06F4
 */
public final class TransformationProviderChain {

	/** Stores the provider list. */
	private final List<ITransformationProvider> providerList =
			new LinkedList<ITransformationProvider>();

	/** Constructor. */
	public TransformationProviderChain(List<ITransformationProvider> providers) {
		Assert.isNotNull(providers);
		Assert.isTrue(!providers.isEmpty());

		this.providerList.addAll(providers);
	}

	/** Performs the transformation. */
	public Object transform(Object source, ITransformationContext context)
			throws ChainTransformationFailedException {
		List<Object> results = new LinkedList<Object>();
		for(ITransformationProvider provider : providerList) {
			if(!provider.canTransform(source, context)) {
				throw new ChainTransformationFailedException(this, provider, results, null);
			}
			try {
				source = provider.transform(source, context);
			} catch(TransformationFailedException e) {
				throw new ChainTransformationFailedException(this, provider, results, e);
			}
			results.add(source);
		}
		return source;
	}
}
