/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.service.base;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.internal.ConnectionCompositorService;

/**
 * Service base implementation, which supports handler registration with set of {@link EObject}
 * classes.
 * 
 * <p>
 * Sub-classes must implement:
 * <UL>
 * <LI>{@link #getExtensionPointName()} to provide the extension point ID,
 * <LI>{@link #getConfigurationElementName()} to provide the configuration element ID,
 * <LI>{@link #getHandlerClassAttribute()} to provide the attribute of the handler class.
 * </UL>
 * 
 * <p>
 * Sub-classes may override:
 * <UL>
 * <LI>{@link #getClassConfigurationElement()} to provide a element class configuration element,
 * <LI>{@link #getClassAttribute()} to provide a element class attribute.
 * <LI> {@link #requiresUniqueHandler()} should return true if each EObject class must not have more
 * than one handler. The default is <code>false</code>.
 * </UL>
 * 
 * An example implementation of a service based on this class is given by
 * {@link ConnectionCompositorService}. The corresponding extension point schema is defined in
 * <code>schema/modelElementConnectionCompositor.exsd</code>.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: A4603AE7A0211B1409E44C57BD495004
 */
public abstract class EObjectAwareServiceBase<T extends IEObjectAware<? extends EObject>> extends
		ObjectAwareServiceBase<T> {

	/**
	 * Returns the configuration element name for the model element class.
	 * Sub-classes may override, but should first consider to use the default
	 * value <i>modelElementClass</i> in the extension point definition.
	 */
	@Override
	protected String getClassConfigurationElement() {
		return "modelElementClass";
	}

	/**
	 * Returns the attribute name for the model element class. Sub-classes may
	 * override, but should first consider to use the default value
	 * <i>modelElementClass</i> in the extension point definition.
	 */
	@Override
	protected String getClassAttribute() {
		return "modelElementClass";
	}
}
